@@ -1,4 +1,4 @@
-# Copyright (C) 2023 Hibiki AI Limited <info@hibiki-ai.com>
+# Copyright (C) 2023-2024 Hibiki AI Limited <info@hibiki-ai.com>
 #
 # This file is part of mirai.promises.
 #
@@ -37,36 +37,16 @@
 #'     Package authors wishing to use the S3 methods may simply import the
 #'     function \code{\link{polling}} to make them available.
 #'
-#' @section Links:
-#'
-#'     \CRANpkg{mirai} website: \url{https://shikokuchuo.net/mirai/}
-#'
-#'     \CRANpkg{nanonext} website: \url{https://shikokuchuo.net/nanonext/}
-#'
-#'     NNG website: \url{https://nng.nanomsg.org/}
-#'
 #' @encoding UTF-8
 #' @author Charlie Gao \email{charlie.gao@@shikokuchuo.net}
 #'     (\href{https://orcid.org/0000-0002-0750-061X}{ORCID})
 #'
 #' @importFrom nanonext is_error_value unresolved
-#' @importFrom promises as.promise promise
+#' @importFrom promises as.promise is.promising promise then
 #'
-#' @docType package
-#' @name mirai.promises-package
-#'
-NULL
+"_PACKAGE"
 
-# nocov start
-# tested implicitly
-
-.onLoad <- function(libname, pkgname) {
-
-  as.promise.recvAio <<- as.promise.mirai <<- as.promise.mirai()
-
-}
-
-# nocov end
+. <- list2env(list(later = .getNamespace("later")[["later"]], pollfreq = 0.1))
 
 #' Make 'Mirai' 'Promise'
 #'
@@ -96,27 +76,51 @@ NULL
 #' @export
 #'
 as.promise.mirai <- function(x) {
-  later <- .getNamespace("later")[["later"]]
-  pollfreq <- 0.1
-  function(x)
-    promise(
+  force(x)
+  then(
+    promise = promise(
       function(resolve, reject) {
         query <- function()
           if (unresolved(x))
-            later(query, delay = pollfreq) else
-              if (is_error_value(value <- .subset2(x, "value")))
-                reject(value) else
-                  resolve(value)
+            .[["later"]](query, delay = .[["pollfreq"]]) else
+              resolve(.subset2(x, "value"))
         query()
       }
-    )
+    ),
+    onFulfilled = function(value)
+      if (is_error_value(value) && !mirai::is_mirai_interrupt(value))
+        stop(value) else
+          value
+  )
 }
 
 #' @rdname as.promise.mirai
 #' @method as.promise recvAio
 #' @export
 #'
-as.promise.recvAio <- as.promise.mirai
+as.promise.recvAio <- function(x) {
+  force(x)
+  then(
+    promise = promise(
+      function(resolve, reject) {
+        query <- function()
+          if (unresolved(x))
+            .[["later"]](query, delay = .[["pollfreq"]]) else
+              resolve(.subset2(x, "value"))
+        query()
+      }
+    ),
+    onFulfilled = function(value)
+      if (is_error_value(value))
+        stop(value) else
+          value
+  )
+}
+
+#' @method is.promising recvAio
+#' @export
+#'
+is.promising.recvAio <- function(x) TRUE
 
 #' Set Polling Frequency
 #'
@@ -139,7 +143,7 @@ as.promise.recvAio <- as.promise.mirai
 polling <- function(freq = 100L) {
 
   is.numeric(freq) || stop("'freq' must be a numeric value")
-  `[[<-`(environment(as.promise.mirai), "pollfreq", freq / 1000L)
+  `[[<-`(., "pollfreq", freq / 1000L)
   invisible()
 
 }
